@@ -226,7 +226,7 @@ expand(const char *cp,	/* input word */
 				quote = 1;
 				continue;
 			  case CQUOTE:
-				quote = 0;
+				quote = st->quote;	/* XXX correct? */
 				continue;
 			  case COMSUB:
 				tilde_ok = 0;
