@@ -19,6 +19,7 @@
 #define SHEREDELIM	8	/* parsing <<,<<- delimiter */
 #define SHEREDQUOTE	9	/* parsing " in <<,<<- delimiter */
 #define STBRACE		10	/* parsing ${..[#%]..} */
+#define SQBRACE		11	/* inside "${}" */
 
 
 /* Structure to keep track of the lexing state and the various pieces of info
@@ -211,16 +212,19 @@ yylex(int cf)
 		  Sbase2:	/* doesn't include *(...|...) pattern (*+?@!) */
 			switch (c) {
 			case '\\':
+ getsc_qchar:
 				c = getsc();
 				if (c) /* trailing \ is lost */
 					*wp++ = QCHAR, *wp++ = c;
 				break;
 			case '\'':
+ open_ssquote:
 				*wp++ = OQUOTE;
 				ignore_backslash_newline++;
 				PUSH_STATE(SSQUOTE);
 				break;
 			case '"':
+ open_sdquote:
 				*wp++ = OQUOTE;
 				PUSH_STATE(SDQUOTE);
 				break;
@@ -253,6 +257,7 @@ yylex(int cf)
 				}
 				break;
 			case '$':
+ subst_dollar:
 				c = getsc();
 				if (c == '(') /*)*/ {
 					c = getsc();
@@ -282,7 +287,10 @@ yylex(int cf)
 						PUSH_STATE(STBRACE);
 					} else {
 						ungetsc(c);
-						PUSH_STATE(SBRACE);
+						if (state == SDQUOTE)
+							PUSH_STATE(SQBRACE);
+						else
+							PUSH_STATE(SBRACE);
 					}
 				} else if (isalpha(c) || c=='_') {
 					*wp++ = OSUBST;
@@ -311,6 +319,7 @@ yylex(int cf)
 				}
 				break;
 			case '`':
+ subst_gravis:
 				PUSH_STATE(SBQUOTE);
 				*wp++ = COMSUB;
 				statep->ls_sbquote.indquotes = 0;
@@ -331,6 +340,7 @@ yylex(int cf)
 					}
 				break;
 			  default:
+ store_char:
 				*wp++ = CHAR, *wp++ = c;
 			}
 			break;
@@ -445,13 +455,23 @@ yylex(int cf)
 			break;
 
 		case SBRACE:
-			/*{*/
-			if (c == '}') {
-				POP_STATE();
-				*wp++ = CSUBST;
-				*wp++ = /*{*/ '}';
-			} else
-				goto Sbase2;
+			if (c == '\'')
+				goto open_ssquote;
+			/* FALLTHROUGH */
+		case SQBRACE:
+			if (c == '"')
+				goto open_sdquote;
+			else if (c == '\\')
+				goto getsc_qchar;
+			else if (c == '$')
+				goto subst_dollar;
+			else if (c == '`')
+				goto subst_gravis;
+			else if (c != /*{*/ '}')
+				goto store_char;
+			POP_STATE();
+			*wp++ = CSUBST;
+			*wp++ = /*{*/ '}';
 			break;
 
 		case STBRACE:
